@@ -2,22 +2,22 @@ from matplotlib.transforms import Bbox
 from aperturedb import ParallelLoader
 from aperturedb import CSVParser
 
-HEADER_X_POS  = "x_pos"
-HEADER_Y_POS  = "y_pos"
-HEADER_WIDTH  = "width"
+HEADER_X_POS = "x_pos"
+HEADER_Y_POS = "y_pos"
+HEADER_WIDTH = "width"
 HEADER_HEIGHT = "height"
-IMG_KEY_PROP  = "img_key_prop"
-IMG_KEY_VAL   = "img_key_value"
+IMG_KEY_PROP = "img_key_prop"
+IMG_KEY_VAL = "img_key_value"
 
 
 class BBoxDataCSV(CSVParser.CSVParser):
     """
     **ApertureDB BBox Data.**
 
-    This class loads the Bounding Box Data which is present in a csv file,
-    and converts it into a series of aperturedb queries.
+    This class loads the Bounding Box Data which is present in a CSV file,
+    and converts it into a series of ApertureDB queries.
 
-    :::note Is backed by a csv file with the following columns:
+    :::note Is backed by a CSV file with the following columns:
     ``IMG_KEY``, ``x_pos``, ``y_pos``, ``width``, ``height``, ``BBOX_PROP_NAME_1``, ... ``BBOX_PROP_NAME_N``, ``constraint_BBOX_PROP_NAME_1``
     :::
 
@@ -34,7 +34,7 @@ class BBoxDataCSV(CSVParser.CSVParser):
 
     **constraint_BBOX_PROP_NAME_1**: Constraints against specific property, used for conditionally adding a Bounding Box.
 
-    Example csv file::
+    Example CSV file::
 
         img_unique_id,x_pos,y_pos,width,height,type,dataset_id,constraint_dataset_id
         d5b25253-9c1e,257,154,84,125,manual,12345,12345
@@ -57,7 +57,7 @@ class BBoxDataCSV(CSVParser.CSVParser):
 
     """
 
-    def __init__(self, filename, **kwargs):
+    def __init__(self, filename: str, **kwargs):
 
         super().__init__(filename, **kwargs)
 
@@ -77,17 +77,11 @@ class BBoxDataCSV(CSVParser.CSVParser):
         }
 
     def getitem(self, idx):
-        idx = self.df.index.start + idx
-
         q = []
-
         img_id = self.df.loc[idx, self.img_key]
-
-        img_ref = (idx % 99998) + 1
-
         fi = {
             "FindImage": {
-                "_ref": img_ref,
+                "_ref": 1,
                 "unique": True,
                 "constraints": {
                     self.img_key: ["==", img_id],
@@ -103,7 +97,7 @@ class BBoxDataCSV(CSVParser.CSVParser):
 
         rect_attrs = ["x", "y", "width", "height"]
         custom_fields = {
-            "image_ref": img_ref,
+            "image_ref": 1,
             "rectangle": {
                 attr: val for attr, val in zip(rect_attrs, box_data)
             },
@@ -123,7 +117,7 @@ class BBoxDataCSV(CSVParser.CSVParser):
 
         return q, []
 
-    def validate(self):
+    def validate(self) -> None:
 
         self.header = list(self.df.columns.values)
 
