@@ -1,3 +1,4 @@
+from typing import Set
 import pandas as pd
 import logging
 from aperturedb.Subscriptable import Subscriptable
@@ -9,10 +10,10 @@ import re
 
 logger = logging.getLogger(__name__)
 
-ENTITY_CLASS      = "EntityClass"
+ENTITY_CLASS = "EntityClass"
 CONSTRAINTS_PREFIX = "constraint_"
 DATE_PREFIX = "date:"
-PROPERTIES  = "properties"
+PROPERTIES = "properties"
 CONSTRAINTS = "constraints"
 
 # This number is based on the partitions one wants to use per core.
@@ -30,11 +31,11 @@ class CSVParser(Subscriptable):
     - **Dask Mode**: This mode is used when the CSV file is too big to fit in memory, or multiprocessing is desired.
         It reads the CSV file into a Dask DataFrame.
         In Dask mode the CSV file is read in chunks, and the operations are performed on each chunk.
-        The tricky bit is that the chunck size is not known till the loader is created, so the processing happens when ingest is called.
+        The tricky bit is that the chunk size is not known till the loader is created, so the processing happens when ingest is called.
         So the Data CSV has another signature, where the df is passed explicitly.
 
     Typically, the response_handler is application specific, and loading does not break
-    on errors in response_handlers, so the default behaviour is to log the error and continue.
+    on errors in response_handlers, so the default behavior is to log the error and continue.
     If you want to break on errors, set strict_response_validation to True.
     """
 
@@ -73,7 +74,7 @@ class CSVParser(Subscriptable):
                     f"CSV file too small to be read in parallel. Use normal mode. cpus: {cpus}")
             self.df = dataframe.read_csv(
                 self.filename,
-                blocksize = blocksize)
+                blocksize=blocksize)
 
         # len for dask dataframe needs a client.
         if not self.use_dask and len(self.df) == 0:
@@ -86,7 +87,7 @@ class CSVParser(Subscriptable):
     def __len__(self):
         return len(self.df.index)
 
-    def get_indexed_properties(self):
+    def get_indexed_properties(self) -> Set[str]:
         if self.constraints_keys:
             return {self._parse_prop(k)[0] for k in self.constraints_keys}
         return set()
