@@ -5,21 +5,21 @@ from aperturedb.Query import QueryBuilder, ObjectType
 logger = logging.getLogger(__name__)
 
 CONNECTION_CLASS = "ConnectionClass"
-PROPERTIES  = "properties"
+PROPERTIES = "properties"
 CONSTRAINTS = "constraints"
 
 
 class ConnectionDataCSV(CSVParser):
     """**ApertureDB Connection Data.**
 
-    This class loads the Connection Data which is present in a csv file,
+    This class loads the Connection Data which is present in a CSV file,
     and converts it into a series of aperturedb queries.
 
-    :::note Is backed by a csv file with the following columns:
+    :::note Is backed by a CSV file with the following columns:
     ``ConnectionClass``, ``Class1@PROP_NAME``, ``Class2@PROP_NAME`` ... ``PROP_NAME_N``, ``constraint_PROP1``
     :::
 
-    Example csv file::
+    Example CSV file::
 
         ConnectionClass,_Image@id,_Descriptor@UUID,confidence,id,constraint_id
         has_image,321423532,AID-0X3E,0.4354,5432543254,5432543254
@@ -28,9 +28,9 @@ class ConnectionDataCSV(CSVParser):
 
     **ConnectionClass**: Arbitrary class name for the entity this would be saved as.
 
-    **``ClassName``@``PropertyName``**: This is a special combination of Class Name and Property Name that can uniquely identify an entity. ‘@’ is a delimeter, so should not be used in a property name.
+    **``ClassName``@``PropertyName``**: This is a special combination of Class Name and Property Name that can uniquely identify an entity. ‘@’ is a delimiter, so should not be used in a property name.
 
-    **PROP_NAME_1 .. PROP_NAME_N**: Arbitraty property names.
+    **PROP_NAME_1 .. PROP_NAME_N**: Arbitrary property names.
 
     **constraint_PROP_NAME_1**: A equality check against a unique property to ensure duplicates are not inserted.
 
@@ -45,7 +45,7 @@ class ConnectionDataCSV(CSVParser):
 
 
     :::info
-    This example csv's first row creates connection between an Image(id=321423532) and a descriptor(UUID=AID-0X3E)
+    This example CSV's first row creates connection between an Image(id=321423532) and a descriptor(UUID=AID-0X3E)
     It also connects the images and descriptors in the subsequent rows.
 
     In the above example, the constraint_id ensures that a connection with the specified
@@ -53,21 +53,21 @@ class ConnectionDataCSV(CSVParser):
     :::
     """
 
-    def __init__(self, filename, **kwargs):
+    def __init__(self, filename: str, **kwargs):
         super().__init__(filename, **kwargs)
 
-        self.props_keys       = [x for x in self.header[3:]
-                                 if not x.startswith(CONSTRAINTS_PREFIX)]
+        self.props_keys = [x for x in self.header[3:]
+                           if not x.startswith(CONSTRAINTS_PREFIX)]
 
         self.constraints_keys = [x for x in self.header[3:]
                                  if x.startswith(CONSTRAINTS_PREFIX)]
 
-        self.src_class   = self.header[1].split("@")[0]
-        self.src_key     = self.header[1].split("@")[1]
-        self.dst_class   = self.header[2].split("@")[0]
+        self.src_class = self.header[1].split("@")[0]
+        self.src_key = self.header[1].split("@")[1]
+        self.dst_class = self.header[2].split("@")[0]
         # Pandas appends a .n to the column name if there is a duplicate
-        self.dst_key     = self.header[2].split("@")[1].split(".")[0]
-        self.command     = "AddConnection"
+        self.dst_key = self.header[2].split("@")[1].split(".")[0]
+        self.command = "AddConnection"
 
     def get_indices(self):
         return {
@@ -77,7 +77,6 @@ class ConnectionDataCSV(CSVParser):
         }
 
     def getitem(self, idx):
-        idx = self.df.index.start + idx
         src_value = self.df.loc[idx, self.header[1]]
         dst_value = self.df.loc[idx, self.header[2]]
         connection_class = self.df.loc[idx, CONNECTION_CLASS]
@@ -85,7 +84,7 @@ class ConnectionDataCSV(CSVParser):
         members = ["_Image", "_Blob", "_Video", "_Descriptor"]
 
         try:
-            ref_src = (2 * idx) % 99998 + 1
+            ref_src = 1
             cmd_params = {
                 "_ref": ref_src,
                 "unique": True,
@@ -98,7 +97,7 @@ class ConnectionDataCSV(CSVParser):
                 cmd_params["blobs"] = False
             q.append(QueryBuilder.find_command(self.src_class, cmd_params))
 
-            ref_dst = ref_src + 1
+            ref_dst = 2
             cmd_params = {
                 "_ref": ref_dst,
                 "unique": True,
