@@ -42,33 +42,9 @@ logger = logging.getLogger(__name__)
 PROTOCOL_VERSION = 1
 
 
-class UnauthorizedException(Exception):
-    pass
-
-
-@dataclass
-class Session():
-
-    session_token:      str
-    refresh_token:      str
-    session_token_ttl:  int
-    refresh_token_ttl:  int
-    session_started:    time.time = time.time()
-
-    def valid(self) -> bool:
-        session_age = time.time() - self.session_started
-
-        # This triggers refresh if the session is about to expire.
-        if session_age > self.session_token_ttl - \
-                int(os.getenv("SESSION_EXPIRTY_OFFSET_SEC", 10)):
-            return False
-
-        return True
-
-
 class ConnectorRest(Connector):
     """
-    **Class to use aperturedb's REST interface**
+    **Class to use ApertureDB's REST interface**
 
     Args:
         str (host): Address of the host to connect to.
@@ -83,29 +59,21 @@ class ConnectorRest(Connector):
                  user="", password="", token="",
                  use_ssl=True, shared_data=None,
                  config: Configuration = None):
+        self.use_keepalive = False
+        super().__init__(
+            host=host,
+            port=port,
+            user=user,
+            password=password,
+            token=token,
+            use_ssl=use_ssl,
+            shared_data=shared_data,
+            config=config)
 
-        if config is None:
-            self.host = host
-
-            if port is None:
-                self.port = 443 if use_ssl else 80
-            else:
-                self.port = port
-
-            self.use_ssl = use_ssl
-            self.config = Configuration(
-                host=self.host,
-                port=self.port,
-                use_ssl=self.use_ssl,
-                username=user,
-                password=password,
-                name="runtime"
-            )
+        if port is None:
+            self.port = 443 if use_ssl else 80
         else:
-            self.config = config
-            self.host = config.host
-            self.port = config.port
-            self.use_ssl = config.use_ssl
+            self.port = port
 
         self.connected = False
         # Session is useful because it does not add "Connection: close header"
@@ -119,19 +87,10 @@ class ConnectorRest(Connector):
         self.url = ('https' if self.use_ssl else 'http') + \
             '://' + host + ':' + str(port) + '/api/'
 
-        if shared_data is None:
-            self.shared_data = SimpleNamespace()
-            self.shared_data.session = None
-            self.shared_data.lock = Lock()
-            try:
-                self._authenticate(user, password, token)
-            except Exception as e:
-                raise Exception("Authentication failed:", str(e))
-        else:
-            self.shared_data = shared_data
+        self.token = token
 
     def __del__(self):
-        logger.info("Done with connector")
+        logger.info("Done with connector REST.")
         self.http_session.close()
 
     def _query(self, query, blob_array = [], try_resume=True):
@@ -150,7 +109,7 @@ class ConnectorRest(Connector):
             files.append(('blobs', blob))
 
         # Set Auth token, only when not authenticated before
-        if self.shared_data.session:
+        if self.shared_data.session and self.shared_data.session.valid():
             headers = {'Authorization': "Bearer " +
                        self.shared_data.session.session_token}
         else:
@@ -158,26 +117,39 @@ class ConnectorRest(Connector):
         tries = 0
         response = SimpleNamespace()
         response.status_code = 0
-        while tries < 3:
+        while tries < self.config.retry_max_attempts:
             tries += 1
-            response = self.http_session.post(self.url,
-                                              headers = headers,
-                                              files   = files,
-                                              verify  = self.use_ssl)
-            if response.status_code == 200:
-                # Parse response:
-                json_response       = json.loads(response.text)
-                import base64
-                response_blob_array = [base64.b64decode(
-                    b) for b in json_response['blobs']]
-                self.last_response  = json_response["json"]
-                break
-            logger.error(
-                f"Response not OK = {response.status_code} {response.text[:1000]}\n\
-                    attempt [{tries}/3] .. PID = {os.getpid()}")
-            time.sleep(1)
+            try:
+                response = self.http_session.post(self.url,
+                                                  headers = headers,
+                                                  files   = files,
+                                                  verify  = self.use_ssl)
+                if response.status_code == 200:
+                    # Parse response:
+                    json_response       = json.loads(response.text)
+                    import base64
+                    response_blob_array = [base64.b64decode(
+                        b) for b in json_response['blobs']]
+                    self.last_response  = json_response["json"]
+                    break
+                logger.error(
+                    f"Response not OK = {response.status_code} {response.text[:1000]}\n\
+                        attempt [{tries}/3] .. PID = {os.getpid()}")
+            except Exception as e:
+                logger.error(f"Exception during http.post = {e}\n\
+                        attempt [{tries}/3] .. PID = {os.getpid()}")
 
-        if tries == 3:
+            time.sleep(self.config.retry_interval_seconds)
+
+            self.connect()
+            if try_resume:
+                self._renew_session()
+
+        if tries == self.config.retry_max_attempts:
             raise Exception(
-                f"Could not query apertureDB {self.config} using REST.")
+                f"Could not query ApertureDB {self.config} using REST.")
         return (self.last_response, response_blob_array)
+
+    def _connect(self):
+        logger.info("Connecting to ApertureDB using REST")
+        self.connected = True
