@@ -2,25 +2,25 @@ import ast
 from aperturedb import ParallelLoader
 from aperturedb import CSVParser
 
-HEADER_NAME   = "name"
-HEADER_DIM    = "dimensions"
+HEADER_NAME = "name"
+HEADER_DIM = "dimensions"
 HEADER_ENGINE = "engine"
 HEADER_METRIC = "metric"
-PROPERTIES    = "properties"
-CONSTRAINTS   = "constraints"
+PROPERTIES = "properties"
+CONSTRAINTS = "constraints"
 
 
 class DescriptorSetDataCSV(CSVParser.CSVParser):
     """**ApertureDB DescriptorSet Data.**
 
-    This class loads the Descriptor Set Data which is present in a csv file,
+    This class loads the Descriptor Set Data which is present in a CSV file,
     and converts it into a series of aperturedb queries.
 
-    :::note Is backed by a csv file with the following columns:
+    :::note Is backed by a CSV file with the following columns:
     ``name``, ``dimensions``, ``engine``, ``metric``, ``PROP_NAME_N``, ``constraint_PROP1``
     :::
 
-    Example csv file::
+    Example CSV file::
 
         name,dimensions,engine,metric
         dining_chairs,2048,FaissIVFFlat,L2
@@ -40,13 +40,13 @@ class DescriptorSetDataCSV(CSVParser.CSVParser):
 
     :::info
     In the above example, the first row implies to create a Descriptor set called dining_chairs.
-    The Descriptors in that set wouldb be expected to be an array of float64, of length 2048.
+    The Descriptors in that set would be expected to be an array of float64, of length 2048.
     When performing a search on this set, FaissIVFFlat engine would be used and the metric to compute
     the distance would be L2.
     :::
     """
 
-    def __init__(self, filename, **kwargs):
+    def __init__(self, filename: str, **kwargs):
 
         super().__init__(filename, **kwargs)
 
