@@ -1,5 +1,6 @@
 
 from aperturedb import ParallelQuery
+from aperturedb.Connector import Connector
 
 import numpy as np
 import logging
@@ -11,11 +12,11 @@ class ParallelLoader(ParallelQuery.ParallelQuery):
     **Parallel and Batch Loader for ApertureDB**
 
     This takes a dataset (which is a collection of homogeneous objects)
-    or a derived class, and optimally inserts them into databse by splitting them
+    or a derived class, and optimally inserts them into database by splitting them
     into batches, and passing the batches to multiple workers.
     """
 
-    def __init__(self, db, dry_run=False):
+    def __init__(self, db: Connector, dry_run: bool = False):
         super().__init__(db, dry_run=dry_run)
         self.type = "element"
 
@@ -35,7 +36,7 @@ class ParallelLoader(ParallelQuery.ParallelQuery):
                                 cls_name, set()).add(prop_name)
         return existing_indices
 
-    def create_indices(self, indices):
+    def create_indices(self, indices) -> None:
         if len(indices) == 0:
             return
 
@@ -68,11 +69,11 @@ class ParallelLoader(ParallelQuery.ParallelQuery):
                 "Failed to create indices; ingestion will be slow.")
             logger.warning(res)
 
-    def query_setup(self, generator):
+    def query_setup(self, generator) -> None:
         if hasattr(generator, "get_indices"):
             self.create_indices(generator.get_indices())
 
-    def ingest(self, generator, batchsize=1, numthreads=4, stats=False):
+    def ingest(self, generator, batchsize: int = 1, numthreads: int = 4, stats: bool = False) -> None:
         """
         **Method to ingest data into the database**
 
@@ -86,7 +87,7 @@ class ParallelLoader(ParallelQuery.ParallelQuery):
             f"Starting ingestion with batchsize={batchsize}, numthreads={numthreads}")
         self.query(generator, batchsize, numthreads, stats)
 
-    def print_stats(self):
+    def print_stats(self) -> None:
 
         times = np.array(self.get_times())
         total_queries_exec = len(times)
@@ -95,17 +96,17 @@ class ParallelLoader(ParallelQuery.ParallelQuery):
         print(f"Total time (s): {self.total_actions_time}")
         print(f"Total queries executed: {total_queries_exec}")
 
-        suceeded_queries = sum([stat["suceeded_queries"]
-                                for stat in self.actual_stats])
-        suceeded_commands = sum([stat["suceeded_commands"]
+        succeeded_queries = sum([stat["succeeded_queries"]
                                 for stat in self.actual_stats])
+        succeeded_commands = sum([stat["succeeded_commands"]
+                                  for stat in self.actual_stats])
 
-        if suceeded_queries == 0:
+        if succeeded_queries == 0:
             print("All queries failed!")
 
         else:
             mean = np.mean(times)
-            std  = np.std(times)
+            std = np.std(times)
             tp = 0 if mean == 0 else 1 / mean * self.numthreads
 
             print(f"Avg Query time (s): {mean}")
@@ -123,6 +124,6 @@ class ParallelLoader(ParallelQuery.ParallelQuery):
 
             # TODO this does not take into account that the last
             # batch may be smaller than batchsize
-            print(f"Total inserted elements: {suceeded_queries}")
-            print(f"Total successful commands: {suceeded_commands}")
+            print(f"Total inserted elements: {succeeded_queries}")
+            print(f"Total successful commands: {succeeded_commands}")
         print("=================================================")
