@@ -8,8 +8,6 @@ from aperturedb import Images
 from torch.utils import data
 
 
-DEFAULT_BATCH_SIZE = 50
-
 logger = logging.getLogger(__name__)
 
 
@@ -17,11 +15,11 @@ class ApertureDBDataset(data.Dataset):
     """
     This class implements a PyTorch Dataset for ApertureDB.
     It is used to load images from ApertureDB into a PyTorch model.
-    It can be initialised with a query that will be used to retrieve
+    It can be initialized with a query that will be used to retrieve
     the images from ApertureDB.
     """
 
-    def __init__(self, db, query, label_prop=None, batch_size=DEFAULT_BATCH_SIZE):
+    def __init__(self, db, query, label_prop=None, batch_size=1):
 
         self.db = db.create_new_connection()
         self.query = query
@@ -33,9 +31,6 @@ class ApertureDBDataset(data.Dataset):
         self.batch_end      = 0
         self.label_prop     = label_prop
 
-        self.prev_requested   = -1
-        self.sequence_counter = batch_size
-
         for i in range(len(query)):
 
             name = list(query[i].keys())[0]
@@ -44,7 +39,7 @@ class ApertureDBDataset(data.Dataset):
 
         if self.find_image_idx is None:
             logger.error(
-                "Query error. The query must containt one FindImage command")
+                "Query error. The query must contain one FindImage command")
             raise Exception('Query Error')
 
         if not "results" in self.query[self.find_image_idx]["FindImage"]:
@@ -63,18 +58,6 @@ class ApertureDBDataset(data.Dataset):
 
     def __getitem__(self, index):
 
-        if index == self.prev_requested + 1:
-            self.sequence_counter += 1
-        else:
-            self.sequence_counter = 0
-
-        if self.sequence_counter >= self.batch_size:
-            self.batch_size = self.batch_size
-        else:
-            self.batch_size = 1
-
-        self.prev_requested = index
-
         if index >= self.total_elements:
             raise StopIteration
 
@@ -107,6 +90,9 @@ class ApertureDBDataset(data.Dataset):
         total_batches = math.ceil(self.total_elements / self.batch_size)
         batch_idx     = math.floor(index / self.batch_size)
 
+        if batch_idx >= total_batches:
+            raise Exception("Index out of range")
+
         query  = self.query
         qbatch = query[self.find_image_idx]["FindImage"]["batch"]
         qbatch["batch_size"] = self.batch_size
@@ -131,7 +117,7 @@ class ApertureDBDataset(data.Dataset):
                 r, b = self.db.query(query)
 
             if len(b) == 0:
-                logger.error("index:", index)
+                logger.error(f"index: {index}")
                 raise Exception("No results returned from ApertureDB")
 
             self.batch_images = b
